/*  Representation of one body.  Provides initial state,	*/
/*  drawing routines.  May be used with several views, not	*/
/*  all at the same scale.					*/
/*				Ernest Sibert -- March 1997	*/

/*  Concerning units:						*/
/*  The customary units are  au - earth mass - year , which	*/
/*  are used for positions, velocities, masses of bodies.	*/
/*  On the other hand, this is inconvenient for dimensions of	*/
/*  bodies, e.g., diameters, so for these we use km .		*/
/*  The conversion factor is NBSystems.b2s ("body to system")	*/
/*  Except for the default initialization, the programs don't	*/
/*  really depend on using this particular system of units.	*/

/*  Drawing conventions:  Bodies are commonly drawn as disks	*/
/*  of diameter d0 centered at the body position.  To allow	*/
/*  for special situations, such as planets with rings, the	*/
/*  programs allow bodies to extend d0 from the body position.	*/
/*  This make for easy clipping, without putting it all on the	*/
/*  the java.awt drawing routines.				*/


import java.awt.*;

class Body
 {  static Color bg = Color.black;		/* Background color	*/

   /* The initial (and partly permanent) state of the body:	*/

   public double  X[],		/* initial position	*/
		  V[],		/* initial velocity	*/
		 mass,		/* mass		   	*/
		 diam;		/* diameter (km)   	*/


   String name;		/* Common name	*/

   Color bclr;		/* Body color	*/

   int  icdiam;		/* icon diameter -  0 means body (and trail)	*/
			/* will not be plotted in wide views.		*/


   /* General initializer */

   public void init(String nm, double Xx, double Xy, double Xz,
		      double Vx, double Vy, double Vz,
		      double im, double idm, Color  bc, int icd)
    { X = new double[3];  X[0] = Xx;  X[1] = Xy;  X[2] = Xz;
      V = new double[3];  V[0] = Vx;  V[1] = Vy;  V[2] = Vz;
      mass = im;  diam = idm;
      name = nm;  bclr = bc;  icdiam = icd;
    }



   /* Draw the body with diameter d0; p gives coordinates of center.	*/

   public void drawBody(Graphics gx, Point p, int d0)
    { if ( d0 > 0 )			/* Body should be drawn	*/
       { int r0 = d0/2;
	 gx.setColor(bclr);			/* Body color	*/
	 gx.fillOval(p.x-r0, p.y-r0, d0, d0);	/* Disk		*/
       }
    }


   /* Like drawBody, in current color.  Usually used to erase.	*/

   public void eraseBody(Graphics gx, Point p, int d0)
    { if ( d0 > 0 )			/* Body should be drawn	*/
       { int r0 = d0/2;
 	 gx.fillOval(p.x-r0, p.y-r0, d0, d0);	/* Disk		*/
       }
    }


   /*  Like drawBody above, but erases previously drawn image	*/
   /*  at p0 and draws new image at p, both of same diameter.	*/
   /*  Draws permanent trail if 'trail' .			*/

   public void moveBody(Graphics gx, Point p, Point p0, int d0, boolean trail)
    { if ( d0 > 0 )				/* Body was drawn	*/
       { int r0 = d0/2;
	 gx.setColor(bg);			/* Set background	*/
	 gx.fillOval(p0.x-r0, p0.y-r0, d0, d0);	/* Erase disk		*/
	 gx.setColor(bclr);			/* Body color		*/
         if ( trail )
	    gx.drawRect(p0.x, p0.y, 1, 1);	/* One point	  */
	 gx.fillOval(p.x-r0, p.y-r0, d0, d0);	/* Fill new disk	*/
       }
    }

 }

