/*  Basic solar system simulation with ViewGroups	*/
/*  implementing multiple views.			*/
/*  Uses Greenspan discrete mechanics as implented in	*/
/*  Grnspan.java					*/
/*			Ernest Sibert - March 1997	*/

import java.awt.*;

public class NBApplt extends java.applet.Applet implements Runnable
 {
   int  N;		/* Number of bodies	*/

   Body bodies[];	/* The bodies 		*/

   Rocket NCC312;	/* The spacecraft	*/
   Body rBody;		/* Body object for rocket	*/

   RControl rc;		/* Spacecraft control panel	*/

   ViewGroup viewxyz;	/* Solar system - xy, zx planes	*/

   Grnspan state;	/* Current state, integration	*/

   /*  Applet control items **********************************	*/

   Thread NBThread;		/* Runs the applet		*/

   Object safety;		/* For synchronization		*/

   int delay = 2000;		/* To slow it down (msec.)	*/

   boolean ready = false,	/* To synchronize startup	*/
	 ptrails = false;	/* Draw permanent trails	*/

   String parm;			/* For applet parameters	*/

   int ns = 10,		/* Number of steps between plots	*/
       np = 10,		/* Number of times to plot		*/
       gs =  0,		/* Loop counter for plotting		*/
     ilim = 10;		/* Iteration limit			*/

   double eps = 1.0e-8,	/* Convergence criterion		*/
	   dt = 0.01,	/* Time step				*/
           dtR = 0.0,   /* Rocker Time step */
	throt = 0.00;   /* Throttle setting for rocket.		*/

  Interp itrpl;          /* Interpolator for bodies */

   double Xi0[][];       /* Interpolation results */
   double Xi1[][];         /* Interpolation results */


   /*  Initialize	*/

   public void init()
    {  safety = new Object();
       parm = getParameter("nbsys");
       if ( parm != null)
	{ Body wbodies[];
	  if ( parm.equalsIgnoreCase("mcnk6") )
	   { wbodies = NBSystems.marciniak6(); }
	  else if ( parm.equalsIgnoreCase("AA1994") )
	   { wbodies = NBSystems.AA1994(); }
	  else
	   { wbodies = new Body[1]; }  /* So it's initialized */
	  N = wbodies.length;
	  bodies = new Body[N+1];
	  for (int i = 0; i < N; i++) bodies[i] = wbodies[i];
	}
       System.out.println(parm+" "+N);
       NCC312 = new Rocket(0.0);
       rBody = new Body();  bodies[N] = rBody;
       Body e = bodies[0];  /* Should be Earth */
       System.out.println("Earth: "+e);
       rBody.init("NCC312", e.X[0] + 0.0003, e.X[1], e.X[2],
			    e.V[0], e.V[1] + 0.62867164747054828, e.V[2],
			    0.0, 0.0, Color.orange, 4);
       parm = getParameter("throttle");
       if ( parm != null ) throt = Double.valueOf(parm).doubleValue();
       NCC312.initR(rBody, 1000.0,  600.0, throt,
		    0.4893291049574622*365.2564*365.2564, 4/365.2564,
		    450.0, N);
       setLayout(new FlowLayout(FlowLayout.CENTER));
       viewxyz = new ViewGroup();  add(viewxyz);
       rc = new RControl();  add(rc);
       viewxyz.init(2, 500, 500, 250, N+1);
       viewxyz.addView(1, 2,  3);	/* Standard x-y view	*/
       viewxyz.addView(1, 3, -2);	/* x-z viewed from -y	*/
       state = new Grnspan(N+1, 0.0);
       state.stime = NBSystems.start;	/* Starting time	*/
       installSystem();
       viewxyz.setControls();
       viewxyz.setView(80.0, 0.0, 0.0, 0.0);
       udtR = new Semaphore();
       rc.init(this, NCC312);
       System.out.println(
	  "au->pix: "+viewxyz.scale+"  km->pix: "+viewxyz.bscale);
    /*  Integration parameters */
       parm = getParameter("deltat");
       if ( parm != null ) dt = Double.valueOf(parm).doubleValue();
       parm = getParameter("epsilon");
       if ( parm != null ) eps = Double.valueOf(parm).doubleValue();
       parm = getParameter("limit");
       if ( parm != null ) ilim = Integer.valueOf(parm).intValue();
       state.initInteg(dt, eps, ilim);
      /*  Integration time, plot controls */
       parm = getParameter("nsteps");
       if ( parm != null ) ns = Integer.valueOf(parm).intValue();
       parm = getParameter("nplots");
       if ( parm != null ) np = Integer.valueOf(parm).intValue();
       parm = getParameter("delay");
       if ( parm != null ) delay = Integer.valueOf(parm).intValue();
       parm = getParameter("ptrails");
       if ( parm != null ) ptrails = parm.equalsIgnoreCase("yes");
       /* Prepare to interpolate */
       itrpl = new Interp();
       dtR = dt/ns;  gk = 1;
       itrpl.set_up(ns, dt, dtR);
       NCC312.deltat = dtR;  NCC312.epsilon = eps;  NCC312.limit = ilim;
       Xi0 = new double[N+1][3];
       Xi1 = new double[N+1][3];

       /* Calculate initial rocket acceleration	*/
       rc.cos[0] = 1.0;  rc.cos[1] = 0.0;  rc.cos[2] = 0.0;
       rc.setAtt();
       NCC312.ready(state.X0, state.Mg);
       System.out.println(NCC312.Thr+"   "+NCC312.dm+"   "+NCC312.DPr);
       ready = true;
    }



   /*  Draw complete system (from scratch) using positions in Xs.	*/

   void drawSystem(double Xs[][])
    { Body b;
      synchronized (safety)
       { viewxyz.drawAll(Xs); }
    }


   /*  Install bodies in state, initialize G.				*/

   void installSystem()
    { NBObj ob;		/* Ignored for the moment	*/
      state.G = NBSystems.G;
      for ( int i = 0;  i < N;  i++ )
       { state.installBody( bodies[i]);
	 ob = viewxyz.loadObject(bodies[i]); 
       }
       ob = viewxyz.loadObject(bodies[N]);
       for (int i = 0;  i < 3;  i++) state.X0[N][i] = NCC312.Xr0[i];
    }



   /*  Move bodies to current positions and replot - preliminary.	*/

   void moveSystem(double Xs[][])
    { viewxyz.drawAll(Xs);
    }


   /* Delay for specified time (in msec.) */

   void pause(int delay)
    { if ( delay > 0 )
       {  try { Thread.sleep(delay); }
          catch (InterruptedException ex) { }
       }
    }

   /* Start process	*/

   public void start()
    { while ( !ready  ) pause(200);		/* Wait for init */
      drawSystem(state.X0);
      if ( NBThread == null )
       { NBThread = new Thread(this);
	 NBThread.start(); }
    }


   /* Stop process  */

   public void stop()
    { if ( NBThread != null )
       synchronized (safety)
	{ NBThread.stop();  NBThread = null; }
    }



   /*  Run system ns*np steps, plotting every ns steps.	*/

   public void run()
    { viewxyz.repaint();
      if ( delay > 0) pause(10*delay);
      viewxyz.bodyCenter(0);		/* Center view on earth */
      viewxyz.menus.position.select(bodies[0].name);
      if ( delay > 0) pause(delay);
      viewxyz.rescale(5.0/1024.0);	/* Very close view	*/
      /* printstate(); */
      if ( delay > 0) pause(2*delay);
      for ( gs = 1;  gs <= np;  )
       { advance();
	 drawSystem(state.X0);
	 if ( delay > 0 ) pause(delay);  viewxyz.repaint();
	 /* printstate(); */
       }
      System.out.println("Finished.");
    }

       
 void advanceRoneStep( double Xa[][], double Xb[][] )
  { NCC312.OneRstep(Xa, Xb, state.Mg);
    double RX[];	/* To swap rocket state	*/
    RX = NCC312.Xr0;  NCC312.Xr0 = NCC312.Xr1;  NCC312.Xr1 = RX;
    RX = NCC312.Vr0;  NCC312.Vr0 = NCC312.Vr1;  NCC312.Vr1 = RX;
    for (int i = 0;  i < 3;  i++) Xb[N][i] = NCC312.Xr0[i];
    rc.th.setProp();  rc.st.setStatus(state.stime);
    state.stime += dtR;
  }


  
  Semaphore udtR;	/* Modification to dtR:		*/
			/*   0      - nothing		*/
			/*   u > 0  - longer by 2^u	*/
			/*   u < 0  - shorter by 2^u	*/

  int gi,	/* counter for the interpolation loop	*/
      gk;	/* interp steps for one rocket step	*/


  /*  Advance over next group of time steps	*/
   void advance()
  { 
     double Xa[][], Xb[][], Xw[][];          /* used only as pointers */

     synchronized (safety){ state.oneStep(); }

     synchronized (safety)
      { Xa = state.X0;
        Xb = Xi1;
        Xw = Xi0;
        for ( gi = 1;  gi < ns; gi++ )
         { itrpl.interpolate(state.X0, state.X1, state.V0, state.V1,
                                                    Xb, gi*gk, N);
         
        advanceRoneStep( Xa, Xb ); 
	  //    printState(Xb, state.V0);
 
        drawSystem(Xb);
	if ( delay > 0 ) pause(delay);  viewxyz.repaint();
        Xa = Xb;
        Xb = Xw;
        Xw = Xa; 
	checkstep();
      } 
    }
   synchronized (safety)
    {
      advanceRoneStep( Xa, state.X1 );
	  //  printState(state.X1, state.V1);
      	/* "Swap" X0, X1; V0, V1  */
      Xw = state.X0;  state.X0 = state.X1;  state.X1 = Xw;
      Xw = state.V0;  state.V0 = state.V1;  state.V1 = Xw;
      checkstep();
      gs++;
    }
   }


   /*  Check whether rocket step should be modified, adjust if needed.	*/

   void checkstep()
    { synchronized ( udtR )
       { int v = udtR.value;
	      if ( v > 0 ) stepFaster(v);
         else if ( v < 0 ) stepSlower(v);
       }
    }


   /*  Adjust step for faster if possible,  v > 0	*/

   void stepFaster(int v)
    {      if ( ns == 1 )			/* Can't go faster	*/
	     {  udtR.value = 0; }
      else if ( (gi & 1) == 0 )			/* gi is even; double	*/
	     { gi = gi/2;  gk = 2*gk; ns = ns/2;
	       dtR = 2.0*dtR;  NCC312.deltat = dtR;
	       NCC312.throttle();
	       udtR.value = v - 1;
	     }  /* Otherwise we can't double now */
    }


   /*  Adjust step for slower, v < 0	*/

   void stepSlower(int v)
    { dtR = dtR/2.0;  NCC312.deltat = dtR;  NCC312.throttle();
      gi = 2*gi;  ns = 2*ns;
      if ( gk == 1) 	/* Calculate more interpolation steps.	*/
	itrpl.set_up(ns, dt, dtR);
      else
        gk = gk/2;	/* We have enough interpolation steps.	*/
      udtR.value = v + 1;
    }



   void printState(double X[][], double V[][])
    {  System.out.println("Time: "+state.stime);
       state.printState( X, V );
       System.out.print("R:");
       for (int u = 0;  u < 3;  u++)  System.out.print("  "+NCC312.Xr0[u]);
       System.out.println();  System.out.print("  ");
       for (int u = 0;  u < 3;  u++)  System.out.print("  "+NCC312.Vr0[u]);
       System.out.println();
    }

 }


